/*
    10-31-03
    Copyright Spark Fun Electronics 2003
    
    10-31-03 SerAccel Build 1.1
             XOff = 0x13 
             XOn = 0x11

    11-2-03 Most of the menu system is laid out
             
    11-4-03 Added RB0 interrupt for incoming user commands
    
    11-6-03 Extended the printout to 1.2345 but @ code size at 95%
            
            Moved to using assembly based HUGE division and multiply routines
            Code size back down to 80%
            
            Moving to pure 8MHz internal operations

    11-7-03 Pure 8MHz @ 57600bps

            Re-worked grav calculation

            Getting calibration routine to work
            
            Currently 99% of PIC filled - removing extra configuration menus
            Down to 62% without extra menu junk
            
            Must get EEPROM memory working
            
            EEPROM is fully working - loads and records calibration values
            65% at this point

    11-8-03 Adding code for the ADXL311.
    
    11-12-03 Installed 100ohm power resistor. The values are quite a bit calmer. 
             Let's try averaging slightly. Still huge variances.
             Measuring the noise with an oscope Vcc-GND, over 200mV of noise. Ahhh. The
             Zener regulator is a horrible regulator.
             
             Installed 7805 TO-220 - Sticks off the PCB. Cleaned voltage down to 50mV noise.
             Readings are very very stable.
             
    11-21-03 New PCBs (SerAccel r3 and Enclosure r1). They institute LDO voltage regulator
             with 100Ohm resistor and .1uF decoupling cap for ADXL.
             
             Pin outs for TX, RX, and PWMs, may have changed slightly.
             
    11-22-03 Enclsoure r1 is in the Soap enclosure. Readings are down to +/-3mg. 
             Shortening printout to only Y.xxx
             
    11-24-03 Attempting to increase output to 100Hz @ true .01sec intervals...
             With BigMath routines, 10,478 cycles to calc each axis
             With built in routines (24bit), 3202 cycles to calc each axis

             6898 Cycles to get ALL caculations complete = 3.4ms
             
             So close! Moving to 115200 baud
             Turns out we had plenty of time - decreased delay between characters

    11-26-03 Board transmits only opening screen (Ben's Dell laptop and ES Computer). 
             May not be receiving incoming commands. Installed TI MAX3221.
             
             Changing code....
             
             Gave up on the Max3221 chip. Draw 10mA which causes voltage on port to dip
             to 3.3V. Does not work.
             
             Changing rs_232_in timing to get it to work with other computers.
             
             Instituted the display frequency option.

    1-8-04   Discovered a problem: when the voltage is take on the first Axis, everything is fine.
             When the voltage is taken on the second Axis, it is consistently off. By implementing
             a delay between measurements, the ADC preforms normally. According to the datasheet,
             this may be TACQ or TAD requirements.

    7-14-04  Migrated to 16F88 in 10 minutes. Now need to design Bloader for the 88 w/o using the UART.
    
    7-15-04  Now using Bloader at 9600 bps. Integrated AccuMark firmware into new Display Output type 'Binary'.
    
    11-22-04 Update to SerAccel v3 including new ADXL320 and 20MHz resonator. Attempting an auto-ranger that will
             select between normal +/-2G with higher resolution to +/-5G with larger resolution

    12-12-04 Re-wrote the soft_rs232_out routines for correct 115200bps timing at 20MHz
    
    4-4-05   New v4 PCB with dual sensor footprints. Changed the calibration routine from 2G mode to 5G mode. 
             The ADXL203 would max out the 2G range at +/-1G sensing.

    6-9-05   New v5 PCB with onboard RS232 converter and Tri-Ax sensor. Did away with all software serial - using 
             onboard UART. Sensor uses 3.3V so PIC is 16LF88 @ 10MHz.
    
*/
#define Clock_10MHz
//#define Serial_In_BB   PORTB.0
//#define Serial_Out_BB   PORTB.3
//#define Baud_115200

//#define DEBUG_MODE

#define TRUE 1
#define FALSE 0
#define POWER_ON    1
#define POWER_OFF   0

#define GS1     PORTB.0
#define GS2     PORTB.1
#define SLEEP   PORTB.3

#define GRAVITY 0
#define RAW     1
#define BINARY  2

#define RANGE_15G       0
#define RANGE_2G        1
#define RANGE_4G        2
#define RANGE_6G        3

#define BR9600          64
#define BR19200         31
#define BR38400         15
#define BR57600         10

//Calibration value locations in EEPROM memory
//============================
#define X_MP_HIGH       1
#define X_MP_LOW        2
#define X_1G_HIGH       3
#define X_1G_LOW        4

#define Y_MP_HIGH       5
#define Y_MP_LOW        6
#define Y_1G_HIGH       7
#define Y_1G_LOW        8

#define Z_MP_HIGH       9
#define Z_MP_LOW        10
#define Z_1G_HIGH       11
#define Z_1G_LOW        12

#define D_MODE          13 //Display mode

#define DELAY_AMT_HIGH  14 //Display frequency delay
#define DELAY_AMT_LOW   15

#define R_MODE          16 //Range mode

#define BAUDRATE        17 //Connection speed
//============================

#include "\Pics\c\16F88.h" // Device hardware map
//#include "\Pics\c\int16CXX.H" // Device interrupt definitions

#pragma config |= 0x2902
#pragma origin 4

//Function definitions
//============================
#include "\Pics\code\Delay.c"   // Delays
#include "\Pics\code\16F88\Shizzle\Stdio.c"   // Basic Serial IO

void boot_up(void);

void calibrate(void);
void main_menu(void);
void change_display_mode(void);
void change_display_frequency(void);
void change_range_mode(void);
void change_baud_rate(void);

uns8 onboard_eeread(uns8 e_address);
void onboard_eewrite(uns8 e_data, uns8 e_address);

void print_accel(uns16, uns16, bit);

uns16 check_X(void);
uns16 check_Y(void);
uns16 check_Z(void);

void display_binary(void);
void display_raw(void);
void display_gravity(void);
//============================
//End Function Definitions

//Global Variables
//============================
uns8 display_mode;
uns8 range_mode;
uns16 temp_counter;
uns16 delay_amount;
uns8 baud_rate;

//Calibration values
uns16 x_midpoint;
uns16 x_1g_width;

bit x_sign;

uns16 y_midpoint;
uns16 y_1g_width;

bit y_sign;

uns16 z_midpoint;
uns16 z_1g_width;

bit z_sign;
//============================
//End Global Variables

void main()
{
    delay_us(10);
    
    boot_up();

    if (display_mode == GRAVITY)
        display_gravity();
    else if (display_mode == RAW)
        display_raw();
    else if (display_mode == BINARY)
        display_binary();
    else //Default
        display_gravity();
        
    while(1)
    {
        printf("Please power cycle SerAccel\n\r", 0);
        delay_ms(1000);
    }
    
}//End Main

void display_gravity(void)
{
    while(1) 
    {
        //Clear timer 1 and start it
        TMR1H = 0; TMR1L = 0;
        TMR1ON = 1;
            
        //Read the ADXL - X 
        //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        putc(' ');
        putc('X');
        putc('=');
        temp_counter = check_X();
        
        if (temp_counter > x_midpoint)
        {
            temp_counter = temp_counter - x_midpoint;
            x_sign = 0;
        }
        else
        {
            temp_counter = x_midpoint - temp_counter;
            x_sign = 1;
        }
        print_accel(temp_counter, x_1g_width, x_sign);
        //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        //Read the ADXL - Y 
        //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        putc(' ');
        putc('Y');
        putc('=');
        temp_counter = check_Y();
        if (temp_counter > y_midpoint)
        {
            temp_counter = temp_counter - y_midpoint;
            y_sign = 0;
        }
        else
        {
            temp_counter = y_midpoint - temp_counter;
            y_sign = 1;
        }
        print_accel(temp_counter, y_1g_width, y_sign);
        //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        //Read the ZAxis
        //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        putc(' ');
        putc('Z');
        putc('=');
        temp_counter = check_Z();
        if (temp_counter > z_midpoint)
        {
            temp_counter = temp_counter - z_midpoint;
            z_sign = 0;
        }
        else
        {
            temp_counter = z_midpoint - temp_counter;
            z_sign = 1;
        }
        print_accel(temp_counter, z_1g_width, z_sign);
        //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

        printf("\n\r", 0);

        #ifdef DEBUG_MODE
        uns16 temp_timer;
        temp_timer.high8 = TMR1H;
        temp_timer.low8 = TMR1L;
        #endif
        
        //See if we need to wait
        while(TMR1H != delay_amount.high8);
        while(TMR1L != delay_amount.low8);
        
        TMR1ON = 0;
        
        //Config mode - fires when XOff or Ctrl-S is received from terminal
        if (RCREG == 0x13) main_menu();

        //PORTB.4 ^= 1; //Flash a pin for O-Scope debugging

        #ifdef DEBUG_MODE
        printf(" T=%d", temp_timer);

        printf(" d=%d", delay_amount);
        #endif
    }
    
}

void display_raw(void)
{
    //Correct for incompatible states (Auto ranging not allowed)
    //if(range_mode == RANGE_2G) range_mode = RANGE_2G;
    //else range_mode = RANGE_5G; //Default to 5G
    
    while(1) 
    {
        //Clear timer 1 and start it
        TMR1H = 0; TMR1L = 0;
        TMR1ON = 1;
            
        //Read the XAxis
        temp_counter = check_X();
        printf(" X=%h", temp_counter);

        //Read the YAxis
        temp_counter = check_Y();
        printf(" Y=%h", temp_counter);

        //Read the YAxis
        temp_counter = check_Z();
        printf(" Z=%h", temp_counter);

        printf("\n\r", 0);

        #ifdef DEBUG_MODE
        uns16 temp_timer;
        temp_timer.high8 = TMR1H;
        temp_timer.low8 = TMR1L;
        #endif

        //See if we need to wait
        while(TMR1H != delay_amount.high8);
        while(TMR1L != delay_amount.low8);
        
        TMR1ON = 0;
        
        //Config mode - fires when XOff or Ctrl-S is received from terminal
        if (RCREG == 0x13) main_menu();

        //PORTB.4 ^= 1; //Flash a pin for O-Scope debugging

        #ifdef DEBUG_MODE
        printf(" T=%d", temp_timer);
        #endif
    }
    
}

void display_binary(void)
{
    uns16 x_val, y_val, z_val;
    uns16 sample_number = 0;
    bit start_sampling = 0;

    uns8 data_byte_in;
    
    //Send out ready message
    putc('#'); //Start with 0x01
    putc('R'); //0x20
    putc('$'); //End with 0x03

    sample_number = 0;
    start_sampling = 0;
        
    while(1) 
    {
        //Clear timer 1 and start it
        TMR1H = 0; TMR1L = 0;
        TMR1ON = 1;

        if(start_sampling == 1)
        {
            x_val = check_X();
            y_val = check_Y();
            z_val = check_Z();
            
            sample_number++;
            
            //Debugging
            //=================================================
            //Display overall time
            #ifdef DEBUG_MODE

                //Used to mimick timing of actual output
                putc('#'); //Start with 0x01
                putc('@'); //Reading output starts with 0x22
                putc('1');
                putc('2');
                putc('3');
                putc('4');
                putc('5');
                putc('6');
                putc('7');
                putc('8');
                putc('$'); //End with 0x03

                uns16 temp_timer;
                temp_timer.high8 = TMR1H;
                temp_timer.low8 = TMR1L;
                
                printf(" T=%d", temp_timer);
        
                printf(" #=%h", sample_number);
        
                //Display Z-Axis
                printf(" X=%h", x_val);
        
                //Display Y-Axis
                printf(" Y=%h", y_val);

                //Display Z-Axis
                printf(" Z=%h", z_val);
        
                printf("\n\r", 0);

            #else
                putc('#'); //Start with 0x01
                putc('@'); //Reading output starts with 0x22
                putc(sample_number.high8); //1
                putc(sample_number.low8); //2
                putc(x_val.high8); //3
                putc(x_val.low8);  //4
                putc(y_val.high8); //5
                putc(y_val.low8);  //6
                putc(z_val.high8); //7
                putc(z_val.low8);  //8
                putc('$'); //End with 0x03
                
            #endif
            //=================================================
        }
    
        //Read incoming UART commands
        if(RCIF == 1)
        {        
            data_byte_in = RCREG;
            RCIF = 0;

            if (data_byte_in == 0x13) //Config mode - fires when XOff or Ctrl-S is received from terminal
            {
                main_menu();
                putc('#'); //Start with 0x01
                putc('R'); //0x20
                putc('$'); //End with 0x03
            }
    
            else if (data_byte_in == 'R') //0x10 Ready?
            {
                data_byte_in = 0;
                //Respond with 0x20
                putc('#'); //Start with 0x01
                putc('R'); //0x20
                putc('$'); //End with 0x03
            }
    
            else if (data_byte_in == 'S') //0x12 Start Sampling
            {
                data_byte_in = 0;
                //Respond with 0x20 and start sampling
                putc('#'); //Start with 0x01
                putc('R'); //0x20
                putc('$'); //End with 0x03
                start_sampling = 1;
            }
    
            else if (data_byte_in == 'A') //0x13 Stop Sampling
            {
                data_byte_in = 0;
                //Respond with 0x20 and stop sampling
                putc('#'); //Start with 0x01
                putc('R'); //0x20
                putc('$'); //End with 0x03
                start_sampling = 0;
            }
    
        }
        
        //Timer1 has prescale setup of 1:8 - 1 timer click = 4us
        //See if we need to wait
        while(TMR1H != delay_amount.high8);
        while(TMR1L != delay_amount.low8);

        TMR1ON = 0;

        //PORTB.4 ^= 1; //Flash a pin for O-Scope debugging
    
    }
}

//Uses the force of gravity as a standard
void calibrate(void)
{
    uns16 z_min, z_max, y_min, y_max, x_min, x_max;

    uns16 temp_counter;

    uns8 temp_mode, i, data_byte_in, x;
    
    uns24 total_temp;

    //XAxis Calibration
    //====================================================================
    printf("\n\n\rRotate on X Axis\n\rPress return at Maximum\n\r", 0);
    
    //Start displaying gravity values for the X axis
    RCIF = 0;
    while(1)
    {
        total_temp = 0;
        for(x = 0 ; x < 16 ; x++)
        {
            temp_counter = check_X(); //Find accel and display only one direction
            total_temp += temp_counter;

            delay_ms(1);
        }
        total_temp /= 16;
        temp_counter = total_temp;
        
        printf(" X=%d  \r", temp_counter);
        
        if(RCIF && RCREG == '\r') break;
    }
    x_max = temp_counter;

    printf("\n\nPress return at Minimum\n\r", 0);
    
    //Start displaying gravity values for the Y axis
    RCIF = 0;
    while(1)
    {
        total_temp = 0;
        for(x = 0 ; x < 16 ; x++)
        {
            temp_counter = check_X(); //Find accel and display only one direction
            total_temp += temp_counter;

            delay_ms(1);
        }
        total_temp /= 16;
        temp_counter = total_temp;

        printf(" X=%d  \r", temp_counter);

        if(RCIF && RCREG == '\r') break;
    }
    x_min = temp_counter;

    //Assign values for the different ranges
    x_midpoint = x_max - x_min;
    x_midpoint = x_midpoint / 2;
    x_midpoint = x_midpoint + x_min;
    
    x_1g_width = x_max - x_midpoint;

    onboard_eewrite(x_midpoint.high8, X_MP_HIGH);
    onboard_eewrite(x_midpoint.low8, X_MP_LOW);

    onboard_eewrite(x_1g_width.high8, X_1G_HIGH);
    onboard_eewrite(x_1g_width.low8, X_1G_LOW);
    //====================================================================

    //YAxis Calibration
    //====================================================================
    delay_ms(50);
    printf("\n\n\rRotate on Y Axis\n\rPress return at Maximum\n\r", 0);
    
    //Start displaying gravity values for the Y axis
    RCIF = 0;
    while(1)
    {
        total_temp = 0;
        for(x = 0 ; x < 16 ; x++)
        {
            temp_counter = check_Y(); //Find accel and display only one direction
            total_temp += temp_counter;

            delay_ms(1);
        }
        total_temp /= 16;
        temp_counter = total_temp;

        printf(" Y=%d  \r", temp_counter);

        if(RCIF && RCREG == '\r') break;
    }
    y_max = temp_counter;

    delay_ms(50);
    printf("\n\nPress return at Minimum\n\r", 0);
    
    //Start displaying gravity values for the Y axis
    RCIF = 0;
    while(1)
    {
        total_temp = 0;
        for(x = 0 ; x < 16 ; x++)
        {
            temp_counter = check_Y(); //Find accel and display only one direction
            total_temp += temp_counter;

            delay_ms(1);
        }
        total_temp /= 16;
        temp_counter = total_temp;

        printf(" Y=%d  \r", temp_counter);

        if(RCIF && RCREG == '\r') break;
    }
    y_min = temp_counter;

    //Assign values for the different ranges
    y_midpoint = y_max - y_min;
    y_midpoint = y_midpoint / 2;
    y_midpoint = y_midpoint + y_min;
    
    y_1g_width = y_max - y_midpoint;
    
    onboard_eewrite(y_midpoint.high8, Y_MP_HIGH);
    onboard_eewrite(y_midpoint.low8, Y_MP_LOW);

    onboard_eewrite(y_1g_width.high8, Y_1G_HIGH);
    onboard_eewrite(y_1g_width.low8, Y_1G_LOW);

    //====================================================================

    //ZAxis Calibration
    //====================================================================
    delay_ms(50);
    printf("\n\n\rRotate on Z Axis\n\rPress return at Maximum\n\r", 0);
    
    //Start displaying gravity values for the Z axis
    RCIF = 0;
    while(1)
    {
        total_temp = 0;
        for(x = 0 ; x < 16 ; x++)
        {
            temp_counter = check_Z(); //Find accel and display only one direction
            total_temp += temp_counter;

            delay_ms(1);
        }
        total_temp /= 16;
        temp_counter = total_temp;

        printf(" Z=%d  \r", temp_counter);

        if(RCIF && RCREG == '\r') break;
    }
    z_max = temp_counter;

    delay_ms(50);
    printf("\n\nPress return at Minimum\n\r", 0);
    
    //Start displaying gravity values for the Y axis
    RCIF = 0;
    while(1)
    {
        total_temp = 0;
        for(x = 0 ; x < 16 ; x++)
        {
            temp_counter = check_Z(); //Find accel and display only one direction
            total_temp += temp_counter;

            delay_ms(1);
        }
        total_temp /= 16;
        temp_counter = total_temp;

        printf(" Z=%d  \r", temp_counter);

        if(RCIF && RCREG == '\r') break;
    }
    z_min = temp_counter;

    //Assign values for the different ranges
    z_midpoint = z_max - z_min;
    z_midpoint = z_midpoint / 2;
    z_midpoint = z_midpoint + z_min;
    
    z_1g_width = z_max - z_midpoint;
    
    onboard_eewrite(z_midpoint.high8, Z_MP_HIGH);
    onboard_eewrite(z_midpoint.low8, Z_MP_LOW);

    onboard_eewrite(z_1g_width.high8, Z_1G_HIGH);
    onboard_eewrite(z_1g_width.low8, Z_1G_LOW);
    //====================================================================    

    #ifdef DEBUG_MODE
    printf("\n\r2G: Xmax = %h ", x_max);
    printf("Xmin = %h ", x_min);
    printf("Xmid = %h ", x_midpoint);
    printf("X-1g = %h", x_1g_width);

    //printf("\n\r5G: Xmax = %h ", x_max_5g);
    //printf("Xmin = %h ", x_min_5g);
    //printf("Xmid = %h ", x_midpoint_5g);
    //printf("X-1g = %h", x_1g_width_5g);
    #endif

    printf("\n\rCalibration Complete!\n\r", 0);
    
}

#pragma codepage 1

//Takes various PWM counters and prints acceleration value
void print_accel(uns16 axis_counter, uns16 calibration_width, bit accel_sign)
{
    uns24 a = 0;
    uns8 b;
    
    a = axis_counter;
    a = a * 1000; 
    a = a / calibration_width;

    uns8 ascii_out[4];
    
    b = a / 1000; //1000s
    a = a % 1000;
    ascii_out[0] = b + '0';

    b = a / 100; //100s
    a = a % 100;
    ascii_out[1] = b + '0';

    b = a / 10; //10s
    a = a % 10;
    ascii_out[2] = b + '0';

    ascii_out[3] = a + '0'; //1s

    //Print the final answer
    if(accel_sign) putc('-');
    else putc(' ');

    uns8 i;
    for(i = 0 ; i < 4 ; i++)
    {
        putc(ascii_out[i]);
        if(i == 0) putc('.');
    }
    
}

//Init vars and ports
void boot_up(void)
{
    
    PORTA = 0b.0000.0000;
    TRISA = 0b.0001.1111;  //0 = Output, 1 = Input (VRef+/- on RA2/3)(XAxis on RA0)(YAxis on RA1)(ZAxis on RA4)

    PORTB = 0b.0000.0000;
    TRISB = 0b.0000.0100;  //0 = Output, 1 = Input (RX on RB2)

    //Setup the ADC module
    //=============================================================
    ANSEL = 0b.0001.1111; //Turn pins to digital I/O or Analog input

    //ADCON1 = 0b.1100.0000; //Setup ADC for right justified with VDD/VSS
    ADCON1 = 0b.1111.0000; //Setup ADC for right justified with Vref+/-

    //=============================================================


    //Setup the hardware UART module
    //=============================================================
    SPBRG = 64; //10MHz for 9600 inital communication baud rate
    TXSTA = 0b.0010.0100; //8-bit asych mode, high speed uart enabled
    RCSTA = 0b.1001.0000; //Serial port enable, 8-bit asych continous receive mode

    //Read in the first few character to check for baud rate reset
    uns8 x;
    for(x = 0 ; x < 100 ; x++)
    {
        if (RCREG == 0x14) //Ctrl+t
        {
            onboard_eewrite(BR9600, BAUDRATE); //Reset the EEPROM to 9600bps
            SPBRG = BR9600; //Set the UART to 9600bps

            while(1)
            {
                printf("Baud rate reset to 9600bps - Please power cycle SerAccel\n\r", 0);
                delay_ms(1000);
            }
        }

        delay_ms(1);
    }

    //Remember baud rate
    baud_rate = onboard_eeread(BAUDRATE);
    if (baud_rate < BR57600) baud_rate = BR9600; //Default to 9600bps
    if (baud_rate > BR9600) baud_rate = BR9600; //Default to 9600bps
    SPBRG = baud_rate;
    //=============================================================
    
    //Read Settings from EEPROM
    //================================================================
    //Load calibration value from non volatile memory
    x_midpoint.high8 = onboard_eeread(X_MP_HIGH);
    x_midpoint.low8 = onboard_eeread(X_MP_LOW);

    x_1g_width.high8 = onboard_eeread(X_1G_HIGH);
    x_1g_width.low8 = onboard_eeread(X_1G_LOW);

    y_midpoint.high8 = onboard_eeread(Y_MP_HIGH);
    y_midpoint.low8 = onboard_eeread(Y_MP_LOW);

    y_1g_width.high8 = onboard_eeread(Y_1G_HIGH);
    y_1g_width.low8 = onboard_eeread(Y_1G_LOW);

    z_midpoint.high8 = onboard_eeread(Z_MP_HIGH);
    z_midpoint.low8 = onboard_eeread(Z_MP_LOW);

    z_1g_width.high8 = onboard_eeread(Z_1G_HIGH);
    z_1g_width.low8 = onboard_eeread(Z_1G_LOW);

    //Remember display mode
    display_mode = onboard_eeread(D_MODE);
    if (display_mode > 2) display_mode = GRAVITY; //Default setting

    //Remember display delay
    delay_amount.high8 = onboard_eeread(DELAY_AMT_HIGH);
    delay_amount.low8 = onboard_eeread(DELAY_AMT_LOW);
    if (delay_amount.high8 == 0xFF) delay_amount = 6250; //Preload to 100Hz
    //if (delay_amount.high8 == 0xFF) delay_amount = 12500; //Preload to 50Hz
    //if (delay_amount.high8 == 0xFF) delay_amount = 500; //Preload to 
    
    //Remember range mode
    range_mode = onboard_eeread(R_MODE);
    if (range_mode > RANGE_6G) range_mode = RANGE_15G;
    
    if(range_mode == RANGE_15G) { GS1 = 0; GS2 = 0; }
    if(range_mode == RANGE_2G)  { GS1 = 1; GS2 = 0; }
    if(range_mode == RANGE_4G)  { GS1 = 0; GS2 = 1; }
    if(range_mode == RANGE_6G)  { GS1 = 1; GS2 = 1; }
    //================================================================

    //Setup Timer1 for delay between measurements
    //T1CON = 0b.0011.0000; //Prescale of 1:8 - 1 timer click = 1.6us @ 20MHz
    //T1CON = 0b.0011.0000; //Prescale of 1:8 - 1 timer click = 3.2us @ 10MHz
    T1CON = 0b.0010.0000; //Prescale of 1:4 - 1 timer click = 1.6us @ 10MHz

    //Turn on the MMA7260Q
    SLEEP = POWER_ON;

}    

//Reads e_data from the onboard eeprom at e_address
uns8 onboard_eeread(uns8 e_address)
{
    EEPGD = 0; //Point to EEPROM Memory

    //Do a read
    EEADR = e_address; //Set the address to read
    RD = 1; //Read it
    
    return(EEDATA); //Read that EEPROM value
}    

//Write e_data to the onboard eeprom at e_address
void onboard_eewrite(uns8 e_data, uns8 e_address)
{
    bit temp_GIE = GIE;
    
    EEPGD = 0; //Point to EEPROM data block
    FREE = 0; //Preform write only  

    EEIF = 0; //Clear the write completion intr flag
    EEADR = e_address; //Set the address
    EEDATA = e_data; //Give it the data
    WREN = 1; //Enable EE Writes
    GIE = 0; //Disable Intrs
    
    //Specific EEPROM write steps
    EECON2 = 0x55;
    EECON2 = 0xAA;
    WR = 1;
    //Specific EEPROM write steps

    while(EEIF == 0); //Wait for write to complete
    EEIF = 0; //Clear the write completion intr flag

    WREN = 0; //Disable EEPROM Writes

    GIE = temp_GIE; //Set GIE to its original state
}

//Checks the Acceleration level in the X Direction
uns16 check_X(void)
{
    uns16 counter;
    
    //Read Channel 0 - RA0 for XAxis
    //ADCON0 = 0b.1000.0001; //Select Fosc/32 for 20MHz, and channel RA0/AN0, A/D on
    ADCON0 = 0b.0100.0001; //Select Fosc/16 for 10MHz, and channel RA0/AN0, A/D on
    
    nop2(); nop2();
        
    GO = 1; //Convert to digital

    while(GO == 1);

    counter.high8 = ADRESH;
    counter.low8 = ADRESL;
    
    return(counter); //Exit without doing value adjustments
}

//Checks the Acceleration level in the Y Direction
uns16 check_Y(void)
{
    uns16 counter;
    
    //Read Channel 1 - RA1 for YAxis
    //ADCON0 = 0b.1000.1001; //Select Fosc/32 for 20MHz, and channel RA1/AN1, A/D on
    ADCON0 = 0b.0100.1001; //Select Fosc/16 for 10MHz, and channel RA1/AN1, A/D on
    
    nop2(); nop2();
        
    GO = 1; //Convert to digital

    while(GO == 1);

    counter.high8 = ADRESH;
    counter.low8 = ADRESL;
    
    return(counter);
}

//Checks the Acceleration level in the Z Direction
uns16 check_Z(void)
{
    uns16 counter;
    
    //Read Channel 4 - RA4 for ZAxis
    //ADCON0 = 0b.1010.0001; //Select Fosc/32 for 20MHz, and channel RA4/AN4, A/D on
    ADCON0 = 0b.0110.0001; //Select Fosc/16 for 10MHz, and channel RA4/AN4, A/D on
    
    nop2(); nop2();

    GO = 1; //Convert to digital

    while(GO == 1);

    counter.high8 = ADRESH;
    counter.low8 = ADRESL;
    
    return(counter);
}

void main_menu(void)
{
    uns24 temp_delay;
    uns8 data_byte_in;

    
    while(1)
    {
        printf("\n\rShizzle Build 5.0 w/MMA7260Q", 0);
        printf("\n\rConfiguration Menu:\n\r", 0);
        printf(" [1] Calibrate\n\r", 0);
        printf(" [2] Display Mode (", 0);
        
        if(display_mode == GRAVITY) printf("Gravity", 0);
        if(display_mode == RAW) printf("Raw", 0);
        if(display_mode == BINARY) printf("Binary", 0);
        printf(")\n\r", 0);
        
        uns24 display_freq;
        display_freq = 625000 / delay_amount;
        printf(" [3] Output Frequency (%dHz)\n\r", display_freq);
        
        printf(" [4] Sensor Range (", 0);
        if(range_mode == RANGE_15G) printf("+/-1.5g", 0);
        if(range_mode == RANGE_2G) printf("+/-2g", 0);
        if(range_mode == RANGE_4G) printf("+/-4g", 0);
        if(range_mode == RANGE_6G) printf("+/-6g", 0);
        printf(")\n\r", 0);

        printf(" [5] Baud Rate (", 0);
        if(baud_rate == BR9600) printf("9600bps", 0);
        if(baud_rate == BR19200) printf("19200bps", 0);
        if(baud_rate == BR38400) printf("38400bps", 0);
        if(baud_rate == BR57600) printf("57600bps", 0);
        printf(")\n\r", 0);
        
        printf(" [x] Exit\n\r: ", 0);
        
        RCIF = 0;
        while(RCIF == 0);
        data_byte_in = RCREG;
    
        //Return to main routine
        if (data_byte_in == 'x')
        {
            data_byte_in = 0;
            break;
        }

        //Calibrate
        if (data_byte_in == '1') calibrate();

        //Display Mode switch
        if (data_byte_in == '2') change_display_mode();

        //Change display delay
        if (data_byte_in == '3') change_display_frequency();

        //Change sensor range
        if (data_byte_in == '4') change_range_mode();

        //Change connection speed
        if (data_byte_in == '5') change_baud_rate();
    }

}

//At 10MHz, 1 machine cycle is 400ns.
//With a prescaler of 4 on timer 1
//1 tmr1 click = 1.6us
void change_display_frequency(void)
{
    uns8 decimal_output[6];
    uns8 data_byte_in;

    uns24 display_freq;
    
    printf("\n\rChange Display Frequency:\n\r", 0);
    printf(" [+] Increase\n\r", 0);
    printf(" [-] Decrease\n\r", 0);
    printf(" [x] Exit\n\r", 0);

    display_freq = 625000 / delay_amount;

//    if(display_mode == GRAVITY && display_freq > 300) display_freq = 300;
//    if(display_mode == RAW && display_freq > 481) display_freq = 480;
//    if(display_mode == BINARY && display_freq > 1221) display_freq = 1220;

    //Display delay and adjust as we go...
    while(1)
    {
        printf(": = %dHz ", display_freq); //Print current delay

        RCIF = 0;
        while(RCIF == 0);
        data_byte_in = RCREG;
        
        /*
            if(baud_rate == BR9600)
            {
                if(display_mode == GRAVITY && display_freq <= 260) display_freq++;
                if(display_mode == RAW && display_freq <= 260) display_freq++;
                if(display_mode == BINARY && display_freq <= 844) display_freq++;
            }
            else if (baud_rate == BR19200)
            {
                if(display_mode == GRAVITY && display_freq <= 510) display_freq++;
                if(display_mode == RAW && display_freq <= 533) display_freq++;
                if(display_mode == BINARY && display_freq <= 1704) display_freq++;
            }
            else if (baud_rate == BR38400)
            {
                if(display_mode == GRAVITY && display_freq <= 820) display_freq++;
                if(display_mode == RAW && display_freq <= 1073) display_freq++;
                if(display_mode == BINARY && display_freq <= 3341) display_freq++;
            }
            else if (baud_rate == BR57600)
            {
                if(display_mode == GRAVITY && display_freq <= 1012) display_freq++;
                if(display_mode == RAW && display_freq <= 1564) display_freq++;
                if(display_mode == BINARY && display_freq <= 4784) display_freq++;
            }
        }
        */

        if(data_byte_in == '+') display_freq++;
        if(data_byte_in == '-' && display_freq > 1) display_freq--;

        if(data_byte_in == 'x') break;

        putc('\r');
        
    }

    delay_amount = 625000 / display_freq;

    //Save the user setting
    onboard_eewrite(delay_amount.high8, DELAY_AMT_HIGH);
    onboard_eewrite(delay_amount.low8, DELAY_AMT_LOW);

    //Return to main menu
}

void change_display_mode(void)
{
    uns8 data_byte_in;

    printf("\n\rChange Display Mode to:\n\r", 0);
    printf(" [1] Calculate Gravity Values\n\r", 0);
    printf(" [2] Raw Values\n\r", 0);
    printf(" [3] Binary Mode\n\r", 0);
    printf(" [x] Exit\n\r: ", 0);

    while(1)
    {
        RCIF = 0;
        while(RCIF == 0);
        data_byte_in = RCREG;
    
        if(data_byte_in == 'x') break;

        //Constant Dump
        if (data_byte_in == '1')
        {
            display_mode = GRAVITY;
            break;
        }
    
        //Dump on return
        else if (data_byte_in == '2')
        {
            display_mode = RAW;
            break;
        }
    
        //AccuMark type output
        else if (data_byte_in == '3')
        {
            display_mode = BINARY;
            break;
        }
    
        else
        {
            printf("Invalid selection. Please try again.\n\r", 0);
        }
    }
        
    //Save the user setting
    onboard_eewrite(display_mode, D_MODE);

    //Force user to reboot unit
    if (data_byte_in > '0' && data_byte_in < '4')
    {
        printf("\n\rPlease Detach and Re-Attach SerAccel\n\r", 0);
        while(1);
    }
}

void change_baud_rate(void)
{
    uns8 data_byte_in;

    printf("\n\rChange Baud Rate to:\n\r", 0);
    printf(" [1] 9600bps\n\r", 0);
    printf(" [2] 19200bps\n\r", 0);
    printf(" [3] 38400bps\n\r", 0);
    printf(" [4] 57600bps\n\r", 0);
    printf(" [x] Exit\n\r: ", 0);

    while(1)
    {
        RCIF = 0;
        while(RCIF == 0);
        data_byte_in = RCREG;
    
        if(data_byte_in == 'x') break;

        //9600
        if (data_byte_in == '1')
        {
            baud_rate = BR9600;
            break;
        }
    
        //19200
        else if (data_byte_in == '2')
        {
            baud_rate = BR19200;
            break;
        }
    
        //38400
        else if (data_byte_in == '3')
        {
            baud_rate = BR38400;
            break;
        }

        //57600
        else if (data_byte_in == '4')
        {
            baud_rate = BR57600;
            break;
        }

        else
        {
            printf("Invalid selection. Please try again.\n\r", 0);
        }
    }
        
    //Save the user setting
    onboard_eewrite(baud_rate, BAUDRATE);

    //Force user to reboot unit
    if (data_byte_in > '0' && data_byte_in < '5')
    {
        printf("\n\rPlease Detach SerAccel, Adjust Port Settings, and Re-Attach SerAccel\n\r", 0);
        while(1);
    }
}


void change_range_mode(void)
{
    uns8 data_byte_in;

    printf("\n\rChange Range Mode to:\n\r", 0);
    printf(" [1] 1.5g range\n\r", 0);
    printf(" [2] 2g range\n\r", 0);
    printf(" [3] 4g range\n\r", 0);
    printf(" [4] 6g range\n\r", 0);
    printf(" [x] Exit\n\r: ", 0);

    while(1)
    {
        RCIF = 0;
        while(RCIF == 0);
        data_byte_in = RCREG;

        if(data_byte_in == 'x') break;

        if (data_byte_in == '1')
        {
            GS1 = 0;
            GS2 = 0;
            range_mode = RANGE_15G;
            break;
        }
    
        else if (data_byte_in == '2')
        {
            GS1 = 1;
            GS2 = 0;
            range_mode = RANGE_2G;
            break;
        }

        else if (data_byte_in == '3')
        {
            GS1 = 0;
            GS2 = 1;
            range_mode = RANGE_4G;
            break;
        }

        else if (data_byte_in == '4')
        {
            GS1 = 1;
            GS2 = 1;
            range_mode = RANGE_6G;
            break;
        }
    
        else
        {
            printf("Invalid selection. Please try again.\n\r", 0);
        }
    }
        
    //Save the user setting
    onboard_eewrite(range_mode, R_MODE);

}